# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

modules = \
['compile_dcm2bids_config']
entry_points = \
{'console_scripts': ['compile-dcm2bids-config = compile_dcm2bids_config:main']}

setup_kwargs = {
    'name': 'compile-dcm2bids-config',
    'version': '1.2.0',
    'description': 'Compile Dcm2Bids configuration files into a single config file.',
    'long_description': '# compile-dcm2bids-config\n\nCombine [`dcm2bids`](https://github.com/unfmontreal/Dcm2Bids) config files into a single config file while preserving the integrity of each separate config file\'s various `IntendedFor` fields.\n\n[![PyPI Version](https://img.shields.io/pypi/v/compile-dcm2bids-config.svg)](https://pypi.org/project/compile-dcm2bids-config/) [![codecov](https://codecov.io/gh/andrewrosss/compile-dcm2bids-config/branch/main/graph/badge.svg?token=BrgPPqwxv4)](https://codecov.io/gh/andrewrosss/compile-dcm2bids-config)\n[![Tests](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/test.yaml/badge.svg)](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/test.yaml)\n[![Code Style](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/lint.yaml/badge.svg)](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/lint.yaml)\n[![Type Check](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/type-check.yaml/badge.svg)](https://github.com/andrewrosss/compile-dcm2bids-config/actions/workflows/type-check.yaml)\n\n## Usage\n\n```bash\n$ compile-dcm2bids-config --help\nusage: compile-dcm2bids-config [-h] [-o OUT_FILE] [-v] in_file [in_file ...]\n\nCombine multiple dcm2bids config files into a single config file.\n\npositional arguments:\n  in_file               The JSON config files to combine\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -o OUT_FILE, --out-file OUT_FILE\n                        The file to write the combined config file to. If not specified\n                        outputs are written to stdout.\n  -v, --version         show program\'s version number and exit\n```\n\n## Getting Started\n\nSuppose you have two config files:\n\n**`example/config1.json`:**\n\n```json\n{\n  "descriptions": [\n    {\n      "dataType": "anat",\n      "modalityLabel": "SWI",\n      "criteria": {\n        "SeriesDescription": "*SWI*"\n      }\n    },\n    {\n      "dataType": "fmap",\n      "modalityLabel": "fmap",\n      "criteria": {\n        "SidecarFilename": "*echo-4*"\n      },\n      "IntendedFor": 0\n    }\n  ]\n}\n```\n\n**`example/config2.json`:**\n\n```json\n{\n  "descriptions": [\n    {\n      "dataType": "dwi",\n      "modalityLabel": "dwi",\n      "criteria": {\n        "SeriesDescription": "*DWI*"\n      }\n    },\n    {\n      "dataType": "anat",\n      "modalityLabel": "SWI",\n      "criteria": {\n        "SeriesDescription": "*SWI*"\n      }\n    },\n    {\n      "id": "my-func",\n      "dataType": "func",\n      "modalityLabel": "bold",\n      "customLabels": "task-rest",\n      "criteria": {\n        "SeriesDescription": "rs_fMRI"\n      },\n      "sidecarChanges": {\n        "SeriesDescription": "rsfMRI"\n      }\n    },\n    {\n      "dataType": "fmap",\n      "modalityLabel": "fmap",\n      "criteria": {\n        "SidecarFilename": "*echo-3*"\n      },\n      "IntendedFor": [0, "my-func"]\n    }\n  ]\n}\n```\n\nThen we can combine the two using the following command (outputs are written to stdout by default):\n\n```bash\n$ compile-dcm2bids-config example/config1.json example/config2.json\n{\n  "descriptions": [\n    {\n      "dataType": "anat",\n      "modalityLabel": "SWI",\n      "criteria": {\n        "SeriesDescription": "*SWI*"\n      }\n    },\n    {\n      "dataType": "fmap",\n      "modalityLabel": "fmap",\n      "criteria": {\n        "SidecarFilename": "*echo-4*"\n      },\n      "IntendedFor": 0\n    },\n    {\n      "dataType": "dwi",\n      "modalityLabel": "dwi",\n      "criteria": {\n        "SeriesDescription": "*DWI*"\n      }\n    },\n    {\n      "dataType": "anat",\n      "modalityLabel": "SWI",\n      "criteria": {\n        "SeriesDescription": "*SWI*"\n      }\n    },\n    {\n      "id": "my-func",\n      "dataType": "func",\n      "modalityLabel": "bold",\n      "customLabels": "task-rest",\n      "criteria": {\n        "SeriesDescription": "rs_fMRI"\n      },\n      "sidecarChanges": {\n        "SeriesDescription": "rsfMRI"\n      }\n    },\n    {\n      "dataType": "fmap",\n      "modalityLabel": "fmap",\n      "criteria": {\n        "SidecarFilename": "*echo-3*"\n      },\n      "IntendedFor": [\n        2,\n        "my-func"\n      ]\n    }\n  ]\n}\n```\n\nNotice that the `IntendedFor` fields have been updated appropriately.\n\n## Python API\n\nYou can also use this tool from within python:\n\n```python\nimport json\nfrom pathlib import Path\nfrom pprint import pp\n\nfrom compile_dcm2bids_config import combine_config\n\n\nconfig1 = json.loads(Path("example/config1.json").read_text())\nconfig2 = json.loads(Path("example/config2.json").read_text())\n\nall_together = combine_config([config1, config2])\n\npp(all_together)\n```\n\nThe result being:\n\n```python\n{\'descriptions\': [{\'dataType\': \'anat\',\n                   \'modalityLabel\': \'SWI\',\n                   \'criteria\': {\'SeriesDescription\': \'*SWI*\'}},\n                  {\'dataType\': \'fmap\',\n                   \'modalityLabel\': \'fmap\',\n                   \'criteria\': {\'SidecarFilename\': \'*echo-4*\'},\n                   \'IntendedFor\': 0},\n                  {\'dataType\': \'dwi\',\n                   \'modalityLabel\': \'dwi\',\n                   \'criteria\': {\'SeriesDescription\': \'*DWI*\'}},\n                  {\'dataType\': \'anat\',\n                   \'modalityLabel\': \'SWI\',\n                   \'criteria\': {\'SeriesDescription\': \'*SWI*\'}},\n                  {\'id\': \'my-func\',\n                   \'dataType\': \'func\',\n                   \'modalityLabel\': \'bold\',\n                   \'customLabels\': \'task-rest\',\n                   \'criteria\': {\'SeriesDescription\': \'rs_fMRI\'},\n                   \'sidecarChanges\': {\'SeriesDescription\': \'rsfMRI\'}},\n                  {\'dataType\': \'fmap\',\n                   \'modalityLabel\': \'fmap\',\n                   \'criteria\': {\'SidecarFilename\': \'*echo-3*\'},\n                   \'IntendedFor\': [2, \'my-func\']}]}\n```\n\n## Contributing\n\n1. Have or install a recent version of `poetry` (version >= 1.1)\n1. Fork the repo\n1. Setup a virtual environment (however you prefer)\n1. Run `poetry install`\n1. Run `pre-commit install`\n1. Add your changes (adding/updating tests is always nice too)\n1. Commit your changes + push to your fork\n1. Open a PR\n',
    'author': 'Andrew Ross',
    'author_email': 'andrew.ross.mail@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/andrewrosss/compile-dcm2bids-config',
    'package_dir': package_dir,
    'py_modules': modules,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<4.0',
}


setup(**setup_kwargs)



import argparse
import json
from copy import deepcopy
from dataclasses import dataclass
from dataclasses import field
from io import TextIOWrapper
from pathlib import Path
from typing import Any
from typing import Dict
from typing import Iterator
from typing import List
from typing import Union


__version__ = "1.2.0"


def main():
    parser = _create_parser()
    args = parser.parse_args()

    if hasattr(args, "handler"):
        return args.handler(args)

    parser.print_help()


def _create_parser() -> argparse.ArgumentParser:
    description = "Combine multiple dcm2bids config files into a single config file."
    parser = argparse.ArgumentParser(description=description)
    parser.add_argument(
        "in_file",
        nargs="+",
        type=Path,
        help="The JSON config files to combine",
    )
    parser.add_argument(
        "-o",
        "--out-file",
        type=argparse.FileType("w", encoding="utf8"),
        default="-",
        help="The file to write the combined config file to. If not "
        "specified outputs are written to stdout.",
    )
    parser.add_argument("-v", "--version", action="version", version=__version__)
    parser.set_defaults(handler=_handler)

    return parser


def _handler(args: argparse.Namespace):
    in_files: list[Path] = args.in_file
    out_file: TextIOWrapper = args.out_file
    # load all the config files passed as arguments
    configs = [json.loads(fp.read_text()) for fp in in_files]
    # combine the config files into one config
    combined_config = combine_config(configs)
    # write the combined config file to disk
    with out_file as f:
        # we output like this because json.dump(obj, f) doesn't add a trailing new-line
        f.write(json.dumps(combined_config, indent=2) + "\n")


def combine_config(input_configs: List[Dict[str, Any]]) -> Dict[str, Any]:
    """Combine multiple dcm2bids config dicts into a single config dict.

    Args:
        input_configs (list[dict[str, Any]]): A list of dcm2bids configs (dicts)

    Returns:
        dict[str, Any]: The combined/merged config dict.
    """

    config_collection = ConfigCollection(input_configs)
    return config_collection.combined()


@dataclass
class ConfigCollection:
    configs: List[Dict[str, Any]] = field(default_factory=list)

    def combined(self):
        return {**self.top_level_params(), "descriptions": list(self.descriptions())}

    def top_level_params(self):
        params = {}
        for config in self.configs:
            c = deepcopy(config)
            c.pop("descriptions", None)
            for k, v in c.items():
                if k in params and params[k] != v:
                    raise TopLevelParameterError(k, params[k], v)
                params[k] = v

        return params

    def descriptions(self) -> Iterator[Dict[str, Any]]:
        seen_ids = set()
        offset = 0
        for config in self.configs:
            descriptions: Union[List[Dict[str, Any]], None] = config.get("descriptions")
            if descriptions is None:
                continue
            for description in descriptions:
                desc_id = description.get("id")
                if isinstance(desc_id, str) and desc_id in seen_ids:
                    raise DescriptionIdError(desc_id)
                elif isinstance(desc_id, str):
                    seen_ids.add(desc_id)

                yield update_intended_for(description, offset)

            offset += len(descriptions)


TIntendedFor = Union[int, str, List[Union[int, str]], None]


def update_intended_for(description: Dict[str, Any], offset: int) -> Dict[str, Any]:
    _description = deepcopy(description)
    intended_for: TIntendedFor = _description.get("IntendedFor")
    if intended_for is None:
        return _description
    elif isinstance(intended_for, str):
        _description["IntendedFor"] = intended_for
    elif isinstance(intended_for, int):
        _description["IntendedFor"] = intended_for + offset
    elif isinstance(intended_for, list):
        _intended_for: List[Union[int, str]] = []
        for i in intended_for:
            if isinstance(i, str):
                _intended_for.append(i)
            elif isinstance(i, int):
                _intended_for.append(i + offset)
            else:
                m = f"IntendedFor must be 'int' or 'str'. Found [{_intended_for}]"
                raise ValueError(m)
        _description["IntendedFor"] = _intended_for
    else:
        m = f"IntendedFor must be int, str or (int | str)[]. Found [{intended_for}]"
        raise ValueError(m)

    return _description


class ConfigurationConflictError(ValueError):
    """Conflicting configuration values"""


class TopLevelParameterError(ConfigurationConflictError):
    def __init__(self, parameter, value1, value2):
        self.parameter = parameter
        self.value1 = value1
        self.value2 = value2
        super().__init__(
            f"Cannot reconcile values [{self.value1!r}] and [{self.value2!r}] "
            f"for top-level configuration parameter [{parameter!r}]",
        )


class DescriptionIdError(ConfigurationConflictError):
    def __init__(self, description_id: str):
        self.description_id = description_id
        super().__init__(f"Found multiple descriptions with ID [{description_id!r}]")


if __name__ == "__main__":
    exit(main())
